@@ -192,6 +192,24 @@ panel_background_image_style_get_type (void)
   return type;
 }
 
+static const GEnumValue _panel_theme_variant_values[] = {
+  { PANEL_THEME_VARIANT_SYSTEM, "PANEL_THEME_VARIANT_SYSTEM", "system" },
+  { PANEL_THEME_VARIANT_LIGHT, "PANEL_THEME_VARIANT_LIGHT", "light" },
+  { PANEL_THEME_VARIANT_DARK, "PANEL_THEME_VARIANT_DARK", "dark" },
+  { 0, NULL, NULL }
+};
+
+GType
+panel_theme_variant_get_type (void)
+{
+  static GType type = 0;
+
+  if (!type)
+    type = g_enum_register_static ("PanelThemeVariant", _panel_theme_variant_values);
+
+  return type;
+}
+
 
 /* enumerations from "../gnome-panel/panel-types.h" */
 #include "../gnome-panel/panel-types.h"
