@@ -45,6 +45,7 @@
 #include "panel-applet-factory.h"
 #include "panel-applet-marshal.h"
 #include "panel-applet-enums.h"
+#include "panel-plug-private.h"
 
 /**
  * SECTION:applet
@@ -94,7 +95,6 @@ struct _PanelAppletPrivate {
 	PanelAppletFlags   flags;
 	PanelAppletOrient  orient;
 	char              *background;
-	GtkWidget         *background_widget;
 
 	int                previous_width;
 	int                previous_height;
@@ -915,7 +915,10 @@ panel_applet_position_menu (GtkMenu   *menu,
 	PanelApplet    *applet;
 	GtkAllocation   allocation;
 	GtkRequisition  requisition;
+	GdkDisplay     *display;
+	GdkSeat        *seat;
 	GdkDevice      *device;
+	GdkWindow      *window;
 	GdkScreen      *screen;
 	int             menu_x = 0;
 	int             menu_y = 0;
@@ -926,16 +929,17 @@ panel_applet_position_menu (GtkMenu   *menu,
 
 	applet = PANEL_APPLET (widget);
 
+	display = gtk_widget_get_display (widget);
+	seat = gdk_display_get_default_seat (display);
+	device = gdk_seat_get_pointer (seat);
 	screen = gtk_widget_get_screen (widget);
-	
-	gtk_menu_set_screen(menu, screen);
+	window = gtk_widget_get_window (widget);
+
+	gtk_menu_set_screen (menu, screen);
 
 	gtk_widget_get_preferred_size (GTK_WIDGET (menu), &requisition, NULL);
-	gdk_window_get_origin (gtk_widget_get_window (widget),
-			       &menu_x, &menu_y);
-	device = gdk_device_manager_get_client_pointer (gdk_display_get_device_manager (gtk_widget_get_display (widget)));
-	gdk_window_get_device_position(gtk_widget_get_window (widget), device, &pointer_x, &pointer_y, NULL);
-
+	gdk_window_get_origin (window, &menu_x, &menu_y);
+	gdk_window_get_device_position (window, device, &pointer_x, &pointer_y, NULL);
 	gtk_widget_get_allocation (widget, &allocation);
 
 	menu_x += allocation.x;
@@ -1075,14 +1079,20 @@ panel_applet_button_event (PanelApplet    *applet,
 	socket_window = gtk_plug_get_socket_window (GTK_PLUG (widget));
 
 	if (event->type == GDK_BUTTON_PRESS) {
+		GdkDisplay *display;
+		GdkSeat *seat;
+
 		xevent.xbutton.type = ButtonPress;
 
+		display = gdk_display_get_default ();
+		seat = gdk_display_get_default_seat (display);
+
 		/* X does an automatic pointer grab on button press
 		 * if we have both button press and release events
 		 * selected.
 		 * We don't want to hog the pointer on our parent.
 		 */
-		gdk_device_ungrab (event->device, GDK_CURRENT_TIME);
+		gdk_seat_ungrab (seat);
 	} else {
 		xevent.xbutton.type = ButtonRelease;
 	}
@@ -1198,54 +1208,6 @@ panel_applet_get_request_mode (GtkWidget *widget)
         return GTK_SIZE_REQUEST_WIDTH_FOR_HEIGHT;
 }
 
-static void
-panel_applet_get_preferred_width (GtkWidget *widget,
-                                  int       *minimum_width,
-                                  int       *natural_width)
-{
-        int focus_width = 0;
-
-        GTK_WIDGET_CLASS (panel_applet_parent_class)->get_preferred_width (widget,
-                                                                           minimum_width,
-                                                                           natural_width);
-        if (!panel_applet_can_focus (widget))
-                return;
-
-        /* We are deliberately ignoring focus-padding here to
-         * save valuable panel real estate.
-         */
-        gtk_widget_style_get (widget,
-                              "focus-line-width", &focus_width,
-                              NULL);
-
-        *minimum_width += 2 * focus_width;
-        *natural_width += 2 * focus_width;
-}
-
-static void
-panel_applet_get_preferred_height (GtkWidget *widget,
-                                  int       *minimum_height,
-                                  int       *natural_height)
-{
-        int focus_width = 0;
-
-        GTK_WIDGET_CLASS (panel_applet_parent_class)->get_preferred_height (widget,
-                                                                            minimum_height,
-                                                                            natural_height);
-        if (!panel_applet_can_focus (widget))
-                return;
-
-        /* We are deliberately ignoring focus-padding here to
-         * save valuable panel real estate.
-         */
-        gtk_widget_style_get (widget,
-                              "focus-line-width", &focus_width,
-                              NULL);
-
-        *minimum_height += 2 * focus_width;
-        *natural_height += 2 * focus_width;
-}
-
 static void
 panel_applet_size_allocate (GtkWidget     *widget,
 			    GtkAllocation *allocation)
@@ -1254,27 +1216,18 @@ panel_applet_size_allocate (GtkWidget     *widget,
 	GtkBin        *bin;
 	GtkWidget     *child;
 	int            border_width;
-	int            focus_width = 0;
 	PanelApplet   *applet;
 
 	if (!panel_applet_can_focus (widget)) {
 		GTK_WIDGET_CLASS (panel_applet_parent_class)->size_allocate (widget, allocation);
 	} else {
-		/*
-		 * We are deliberately ignoring focus-padding here to
-		 * save valuable panel real estate.
-		 */
-		gtk_widget_style_get (widget,
-				      "focus-line-width", &focus_width,
-				      NULL);
-
 		border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
 
 		gtk_widget_set_allocation (widget, allocation);
 		bin = GTK_BIN (widget);
 
-		child_allocation.x = focus_width;
-		child_allocation.y = focus_width;
+		child_allocation.x = 0;
+		child_allocation.y = 0;
 
 		child_allocation.width  = MAX (allocation->width  - border_width * 2, 0);
 		child_allocation.height = MAX (allocation->height - border_width * 2, 0);
@@ -1286,9 +1239,6 @@ panel_applet_size_allocate (GtkWidget     *widget,
 						child_allocation.width,
 						child_allocation.height);
 
-		child_allocation.width  = MAX (child_allocation.width  - 2 * focus_width, 0);
-		child_allocation.height = MAX (child_allocation.height - 2 * focus_width, 0);
-
 		child = gtk_bin_get_child (bin);
 		if (child)
 			gtk_widget_size_allocate (child, &child_allocation);
@@ -1311,7 +1261,6 @@ panel_applet_draw (GtkWidget *widget,
 {
         GtkStyleContext *context;
 	int border_width;
-	int focus_width = 0;
 	gdouble x, y, width, height;
 
 	GTK_WIDGET_CLASS (panel_applet_parent_class)->draw (widget, cr);
@@ -1322,14 +1271,6 @@ panel_applet_draw (GtkWidget *widget,
         width = gtk_widget_get_allocated_width (widget);
         height = gtk_widget_get_allocated_height (widget);
 
-	/*
-	 * We are deliberately ignoring focus-padding here to
-	 * save valuable panel real estate.
-	 */
-	gtk_widget_style_get (widget,
-			      "focus-line-width", &focus_width,
-			      NULL);
-
 	border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
 
 	x = 0; // FIXME: border_width ?
@@ -1520,8 +1461,6 @@ panel_applet_get_background (PanelApplet *applet)
 
                 g_variant_get (variant, "(uii)", &xid, &x, &y);
                 pattern = panel_applet_get_pattern_from_pixmap (applet, xid, x, y);
-                if (!pattern)
-                        g_warning ("Failed to get pixmap %d, %d, %d", xid, x, y);
         }
                 break;
         default:
@@ -1563,38 +1502,28 @@ panel_applet_style_updated (GtkWidget *widget)
   panel_applet_handle_background (applet);
 }
 
-static void
-panel_applet_update_background_for_widget (GtkWidget       *widget,
-					   cairo_pattern_t *pattern)
-{
-	GdkWindow *window;
-
-	window = gtk_widget_get_window (widget);
-
-	if (!window)
-		return;
-
-	gdk_window_set_background_pattern (window,
-	                                   pattern);
-}
-
 static void
 panel_applet_handle_background (PanelApplet *applet)
 {
-        cairo_pattern_t *pattern;
+	GdkWindow *window;
+	cairo_pattern_t *pattern;
+
+	if (applet->priv->plug == NULL)
+		return;
+
+	window = gtk_widget_get_window (applet->priv->plug);
+
+	if (window == NULL)
+	  return;
 
 	pattern = panel_applet_get_background (applet);
 
-	if (applet->priv->background_widget)
-		panel_applet_update_background_for_widget (applet->priv->background_widget,
-							   pattern);
-	else
-		panel_applet_update_background_for_widget (GTK_WIDGET (applet), pattern);
+	gdk_window_set_background_pattern (window, pattern);
+	gtk_widget_queue_draw (applet->priv->plug);
 
-        g_signal_emit (G_OBJECT (applet),
-                        panel_applet_signals [CHANGE_BACKGROUND],
-                        0, pattern);
-        if (pattern)
+	g_signal_emit (applet, panel_applet_signals [CHANGE_BACKGROUND], 0, pattern);
+
+	if (pattern)
 		cairo_pattern_destroy (pattern);
 }
 
@@ -1750,7 +1679,7 @@ panel_applet_setup (PanelApplet *applet)
 {
 	GValue   value = {0, };
 	GArray  *params;
-	gint     i;
+	guint    i;
 	gboolean ret;
 
 	g_assert (applet->priv->id != NULL &&
@@ -1793,6 +1722,7 @@ static void
 panel_applet_init (PanelApplet *applet)
 {
 	GtkStyleContext *context;
+	gchar *xml;
 
 	applet->priv = panel_applet_get_instance_private (applet);
 
@@ -1807,7 +1737,7 @@ panel_applet_init (PanelApplet *applet)
 
 	applet->priv->builder = gtk_builder_new ();
 
-	gchar *xml = g_strdup_printf (panel_menu_ui, _("_Remove From Panel"), _("_Move"));
+	xml = g_strdup_printf (panel_menu_ui, _("_Remove From Panel"), _("_Move"));
 	gtk_builder_add_from_string (applet->priv->builder, xml, -1, NULL);
 	g_free (xml);
 
@@ -1822,9 +1752,6 @@ panel_applet_init (PanelApplet *applet)
 	gtk_style_context_add_class (context, GTK_STYLE_CLASS_HORIZONTAL);
 
 	panel_applet_composited_changed (GTK_WIDGET (applet));
-
-	/* FIXME: High dpi scaling does not work... */
-	gdk_x11_display_set_window_scale (gdk_display_get_default (), 1);
 }
 
 static GObject *
@@ -1843,7 +1770,7 @@ panel_applet_constructor (GType                  type,
 	if (!applet->priv->out_of_process)
 		return object;
 
-	applet->priv->plug = gtk_plug_new (0);
+	applet->priv->plug = panel_plug_new ();
 
 	g_signal_connect_swapped (G_OBJECT (applet->priv->plug), "embedded",
 		                      G_CALLBACK (panel_applet_setup),
@@ -1887,8 +1814,6 @@ panel_applet_class_init (PanelAppletClass *klass)
 	widget_class->composited_changed = panel_applet_composited_changed;
 	widget_class->key_press_event = panel_applet_key_press_event;
 	widget_class->get_request_mode = panel_applet_get_request_mode;
-        widget_class->get_preferred_width = panel_applet_get_preferred_width;
-        widget_class->get_preferred_height = panel_applet_get_preferred_height;
 	widget_class->size_allocate = panel_applet_size_allocate;
 	widget_class->draw = panel_applet_draw;
 	widget_class->style_updated = panel_applet_style_updated;
@@ -2091,6 +2016,8 @@ panel_applet_class_init (PanelAppletClass *klass)
 	add_tab_bindings (binding_set, GDK_SHIFT_MASK, GTK_DIR_TAB_BACKWARD);
 	add_tab_bindings (binding_set, GDK_CONTROL_MASK, GTK_DIR_TAB_FORWARD);
 	add_tab_bindings (binding_set, GDK_CONTROL_MASK | GDK_SHIFT_MASK, GTK_DIR_TAB_BACKWARD);
+
+	gtk_widget_class_set_css_name (widget_class, "panel-applet");
 }
 
 static void
@@ -2424,21 +2351,13 @@ panel_applet_factory_setup_in_process (const gchar               *factory_id,
  * Configure #PanelApplet to automatically draw the background of the applet on
  * @widget. It is generally enough to call this function with @applet as
  * @widget.
+ *
+ * Deprecated: 3.20: Do not use this API. Since 3.20 this function does nothing.
  **/
 void
 panel_applet_set_background_widget (PanelApplet *applet,
 				    GtkWidget   *widget)
 {
-	applet->priv->background_widget = widget;
-
-	if (widget && gtk_widget_get_realized (widget)) {
-		cairo_pattern_t *pattern;
-
-		pattern = panel_applet_get_background (applet);
-		panel_applet_update_background_for_widget (widget, pattern);
-		if (pattern)
-			cairo_pattern_destroy (pattern);
-	}
 }
 
 guint32
